import generator
import sqlite3
from flask import Flask, request
from grammar import Grammar
import time
import os

app = Flask(__name__)

gdir = "."
f = open(os.path.join(gdir, 'template.html'))
template = f.read()
f.close()

htmlgrammar = Grammar()
err = htmlgrammar.parse_from_file(os.path.join(gdir, 'html.txt'))
if err > 0:
	print 'Errors'
	exit(0)

cssgrammar = Grammar()
err = cssgrammar.parse_from_file(os.path.join(gdir, 'css.txt'))
if err > 0:
	print 'Errors'
	exit(0)

jsgrammar = Grammar()
err = jsgrammar.parse_from_file(os.path.join(gdir, 'js.txt'))
if err > 0:
	print 'Errors'
	exit(0)

htmlgrammar.add_import('cssgrammar', cssgrammar)
jsgrammar.add_import('cssgrammar', cssgrammar)


def genTests():
	con = sqlite3.connect('testcases.db')
	c = con.cursor()

@app.route("/reset")
def reset():
	os.system("rm testcases.db")

@app.route("/selphish")
def selphish():
	with open('suspicious.log','r') as f:
		domains = f.read()
		html = '''
			<html>
			<head>
			<title>SelPhish | Cycure Labs</title>
			<body>
			<h1>Sel<b>Phish</b> | Cycure Labs</h1>
			<br />
			<p>Keep refreshing the page, to get more phishing links. As it keeps working in the background, It keeps scanning for the phishing domains over the web.</p> <br />
			<h3>Domains: </h3>
			<br />
			{}
			</body>
			</head>
			</html>
		'''.format(domains)
			#r.close()

	#os.rename('selphish.html', '/var/www/html/domfuzzer/domato/selphish.html')
	return html

@app.route("/status")
def status():
	con = sqlite3.connect("testcases.db")
	c = con.cursor()

	buf = '''
<html>
<head><meta http-equiv="refresh" content="1;" /></head>
<body>
<h1>Vexillium | Cycure Labs</h1>
'''

	buf += "Status<br>\n"
	buf += "Fuzzing rate: <br>\n"
	c.execute("select count(*) from iters where time > ?", (time.time() - 60,))
	min1 = c.fetchone()[0]
	c.execute("select count(*) from iters where time > ?", (time.time() - 60*5,))
	min5 = c.fetchone()[0]
	c.execute("select count(*) from iters where time > ?", (time.time() - 60*30,))
	min30 = c.fetchone()[0]
	buf += "%s %s %s<br>\n" % (min1, min5, min30)

	#buf += "Last Crash: %s <br>\n"
	buf += "<br>Crash info:<br>\n"

	c.execute("select time from iters where status=\"crash\" order by time desc limit 1")
	last = c.fetchone()
	if last:
		buf += "Time since last crash: %d seconds <br>\n" % (time.time() - last[0])
		c.execute("select count(*) from iters status=\"crash\"")
		buf += "Total number of crashes: %d <br>\n" % (c.fetchone())

	else:
		buf += "No recorded crashes.\n"
	buf += "<br>Overall status:<br>\n"
	c.execute("select time from iters order by time asc limit 1")
	buf += "Time fuzzing: %0.0d seconds <br>\n" % (time.time() - c.fetchone()[0])
	c.execute("Select count (*) from iters;")
	buf += "Total number of testcases: %d <br>\n" % c.fetchone()

	buf += "</body>\n</html>"

	return buf


@app.route("/fuzz")
def fuzzer():
	con = sqlite3.connect('testcases.db')
	c = con.cursor()
	print 'Connection from %s' % request.remote_addr
	buf = generator.generate_new_sample(template, htmlgrammar, cssgrammar, jsgrammar)

	# Reset testcases
	c.execute("delete from testcases where id in (select testcase from iters where status = \"running\" and source= ?)", (request.remote_addr,))
	c.execute("update iters set status=\"completed\" where status = \"running\" and source = ?", (request.remote_addr,))

	#Set new testcases
	c.execute("insert into iters(time, source, status) values (?, ?, \"running\")", (time.time(), request.remote_addr, ))

	c.execute("insert into testcases(testcase, id) values (?, (select testcase from iters where source = ? and status = \"running\"))", (buf, request.remote_addr))


	con.commit()
	con.close()
	return buf

@app.route("/")
def start_job():
	con = sqlite3.connect("testcases.db")
	c = con.cursor()
	try:
		c.execute("select * from testcases")
	except sqlite3.OperationalError:
		c.execute("create table testcases(id int, testcase text)")
		c.execute("create table iters(time int, source text, testcase integer primary key autoincrement, status text)")
		#c.execute("create table results(id int, testcase text)")
		con.commit()
		con.close()
		print "Table initialised"

	buf = '''
<html>
<head><meta http-equiv="refresh" content="1;" /></head>
<body>
<h1>Vexillium | Cycure Labs</h1>
<iframe src="fuzz" height=600 width=600></iframe>
</body>
</html>
'''
	return buf

if __name__ == "__main__":
	if os.name == 'nt':
		os.system('cls')
	else:
		os.system('clear')

	print 'Vexillium - Browser DOM Fuzzer'
	print '     - Cycure Labs -'
	app.run(threaded=True, host='127.0.0.1', port=1337)
	#for i in xrange(2):
	#	print i
	#	generator.generate_new_sample(template, htmlgrammar, cssgrammar, jsgrammar)
